import os
import sys
import argparse

import core

from .tasks import generate, run, http, test as tasks_test, tarball, mkimg


"""
入口
"""



def cmdline():

    parser = argparse.ArgumentParser(description="插件生成器")

    parser.add_argument("-v", "--version", help="查看版本", action="store_true")
    parser.add_argument("-g", "--generate", help="插件生成", action="append")
    parser.add_argument("-r", "--run", help="运行action", action="append")
    parser.add_argument("-hp", "--http", help="启动api接口", action="store_true")
    parser.add_argument("-t", "--test", help="测试", action="append")
    parser.add_argument("-tb", "--tarball", help="插件打包", action="store_true")
    parser.add_argument("-mki", "--mkimg", help="制作成docker镜像", action="store_true")



    args = parser.parse_args()

    if args.version:
        print(f"chariot-plugin: {core.VERSION}")

    elif args.generate:

        yml = (args.generate)[0]

        generate(os.getcwd(), yml)

    elif args.run:

        path = os.getcwd()
        tests_path = args.run[0]

        run(path, tests_path)

    elif args.http:

        path = os.getcwd()

        http(path)

    elif args.test:

        path = os.getcwd()
        tests_path = args.test[0]

        tasks_test(path, tests_path)

    elif args.tarball:

        path = os.getcwd()
        tarball(path)

    elif args.mkimg:

        path = os.getcwd()
        mkimg(path)










import requests
from typing import Optional
import sys
from pydantic import BaseModel, ValidationError

try:
    from .base import Core, CatchErr
except:
    from base import Core, CatchErr


"""
插件各组件基类
"""


class Actions(object):
    # 类变量　格式化输出
    format_output = {
        "type": "trigger_event",
        "version": "v1",
        "body": {
            "meta": {},
            "output": None,
            "status": "ok",
            "log": "succ",
        }
    }

    log_out = Core().print_to_stdout
    log_err = Core().print_to_stderr

    def __init__(self):

        # action name 按规范应小写且用下划线隔开单词
        self.name = ""

        # 入参　校验类
        self.inputModel = None

        # 出参　校验类
        self.outputModel = None

        # 连接器　校验类
        self.connModel = None





    def connection(self, data: Optional[dict] = None):

        ...


    def run(self, params):
        ...

    @CatchErr.print_err_stack
    def test(self, connect):
        output = self.format_output

        try:
            self.connection(connect)
            output['body']["log"] = f"{self.name} 连接器测试无异常"
        except Exception as e:
            print(f"连接器　测试失败 - {str(e)}", file=sys.stderr)
            output['body']["log"] = f"{self.name} 连接器　测试异常 - {str(e)}"
        finally:
            self.log_out(output)
            return output

    @CatchErr.print_err_stack
    def _run(self, inp, connect):
        """
        私有函数，一切从我运行
        """

        core = Core()

        # 入参验证
        core.check_data(inp, self.inputModel)
        core.check_data(connect, self.connModel)

        # 运行自写　run
        self.connection(connect)
        data = self.run(inp)


        # 出参验证
        core.check_data(data, self.outputModel)

        # 补全output
        output = core.complete_output(data, "succ")

        # 输出到 stdout
        self.log_out(output)

        return output


class Triggers(object):
    # 类变量　格式化输出
    format_output = {
        "type": "trigger_event",
        "version": "v1",
        "body": {
            "meta": {},
            "output": None,
            "status": "ok",
            "log": "succ",
        }
    }
    log_out = Core().print_to_stdout
    log_err = Core().print_to_stderr

    def __init__(self):
        # action name 按规范应小写且用下划线隔开单词
        self.name = ""

        # 发送到
        self.dispatcher_url = ""

        # 入参　校验类
        self.inputModel = None

        # 出参　校验类
        self.outputModel = None

        # 类型
        self.connModel = None



    def connection(self, data: Optional[dict] = None):

        ...

    def run(self, params):
        ...

    @CatchErr.print_err_stack
    def test(self, connect):

        output = self.format_output

        try:
            self.connection(connect)
            output['body']["log"] = f"{self.name} 连接器测试无异常"
        except Exception as e:
            print(f"连接器　测试失败 - {str(e)}", file=sys.stderr)
            output['body']["log"] = f"{self.name} 连接器　测试异常 - {str(e)}"
        finally:
            self.log_out(output)
            return output

    @CatchErr.print_err_stack
    def _run(self, inp, connect, dispatcher_url):
        """
        私有函数，一切从我运行
        """

        self.dispatcher_url = dispatcher_url

        core = Core()

        # 入参验证
        core.check_data(inp, self.inputModel)
        core.check_data(connect, self.connModel)

        # 运行自写　run
        self.connection(connect)
        self.run(inp)

        output = self.format_output
        output["body"]["log"] = f"{self.name} succ"

        self.log_out(output)
        return output

    @CatchErr.print_err_stack
    def send(self, data: dict = {}) -> None:
        core = Core()

        core.check_data(data, self.outputModel)
        self.log_out(data)

        resp = requests.post(self.dispatcher_url, json=data)


class IndicatorReceivers(object):
    # 类变量　格式化输出
    format_output = {
        "type": "indicator_receiver_event",
        "version": "v1",
        "body": {
            "meta": {},
            "output": None,
            "status": "ok",
            "log": "succ",
        }
    }
    log_out = Core().print_to_stdout
    log_err = Core().print_to_stderr

    def __init__(self):
        # action name 按规范应小写且用下划线隔开单词
        self.name = ""

        # 发送到
        self.dispatcher_url = ""

        # 入参　校验类
        self.inputModel = None

        # 出参　校验类
        self.outputModel = None

        # 类型
        self.connModel = None



    def connection(self, data: Optional[dict] = None):

        ...

    def run(self, params):
        ...

    @CatchErr.print_err_stack
    def test(self, connect):

        output = self.format_output

        try:
            self.connection(connect)
            output['body']["log"] = f"{self.name} 连接器测试无异常"
        except Exception as e:
            print(f"连接器　测试失败 - {str(e)}", file=sys.stderr)
            output['body']["log"] = f"{self.name} 连接器　测试异常 - {str(e)}"
        finally:
            self.log_out(output)
            return output

    @CatchErr.print_err_stack
    def _run(self, inp, connect, dispatcher_url):
        """
        私有函数，一切从我运行
        """

        self.dispatcher_url = dispatcher_url

        core = Core()

        # 入参验证
        core.check_data(inp, self.inputModel)
        core.check_data(connect, self.connModel)

        # 运行自写　run
        self.connection(connect)
        self.run(inp)

        output = self.format_output
        output["body"]["log"] = f"{self.name} succ"

        self.log_out(output)
        return output

    @CatchErr.print_err_stack
    def send(self, data: dict = {}) -> None:
        core = Core()

        core.check_data(data, self.outputModel)
        self.log_out(data)

        resp = requests.post(self.dispatcher_url, json=data)

        self.log_out(f"[Response {resp.status_code}] {resp.text}")


class AlarmReceivers(object):
    # 类变量　格式化输出
    format_output = {
        "type": "alarm_receiver_event",
        "version": "v1",
        "body": {
            "meta": {},
            "output": None,
            "status": "ok",
            "log": "succ",
        }
    }
    log_out = Core().print_to_stdout
    log_err = Core().print_to_stderr

    def __init__(self):
        # action name 按规范应小写且用下划线隔开单词
        self.name = ""

        # 发送到
        self.dispatcher_url = ""

        # 入参　校验类
        self.inputModel = None

        # 出参　校验类
        self.outputModel = None

        # 类型
        self.connModel = None



    def connection(self, data: Optional[dict] = None):

        ...

    def run(self, params):
        ...

    @CatchErr.print_err_stack
    def test(self, connect):

        output = self.format_output

        try:
            self.connection(connect)
            output['body']["log"] = f"{self.name} 连接器测试无异常"
        except Exception as e:
            print(f"连接器　测试失败 - {str(e)}", file=sys.stderr)
            output['body']["log"] = f"{self.name} 连接器　测试异常 - {str(e)}"
        finally:
            self.log_out(output)
            return output

    @CatchErr.print_err_stack
    def _run(self, inp, connect, dispatcher_url):
        """
        私有函数，一切从我运行
        """

        self.dispatcher_url = dispatcher_url

        core = Core()

        # 入参验证
        core.check_data(inp, self.inputModel)
        core.check_data(connect, self.connModel)

        # 运行自写　run
        self.connection(connect)
        self.run(inp)

        output = self.format_output
        output["body"]["log"] = f"{self.name} succ"

        self.log_out(output)
        return output

    @CatchErr.print_err_stack
    def send(self, data: dict = {}) -> None:
        core = Core()

        core.check_data(data, self.outputModel)
        self.log_out(data)

        resp = requests.post(self.dispatcher_url, json=data)

        self.log_out(f"[Response {resp.status_code}] {resp.text}")








from fastapi import FastAPI
import typing
import uvicorn
import json


try:
    from . import models
    from .base import Core, CatchErr
except:
    import models
    from base import Core, CatchErr


core = Core()


app = FastAPI(title="TANZE", version="0.1.0", description="自写sdk, 增加 test接口可以测试连接器，连接器并不为单例的，第一执行的组件")

# web 调用　自写方法　关键　
plugins = None


class Server(object):


    def __init__(self, plus):

        global plugins
        plugins = plus

    @staticmethod
    @app.post("/triggers/{trigger_name}")
    def triggers(trigger_name: str, plugin_stdin: typing.Optional[models.PluginStdin]):

        # 外部类
        trigger_model = plugins.triggers[trigger_name]


        # 取出body
        plugin_stdin = json.dumps(plugin_stdin.dict())
        stdin_body = core.parse_stdin(plugin_stdin)

        # 获取input
        inp = core.extract_pointer(stdin_body, "/input")
        connect_data = core.extract_pointer(stdin_body, "/connection")
        dispatcher_url = core.extract_pointer(stdin_body, "/dispatcher/url")


        # 执行　外部run 相关操作
        output = trigger_model._run(inp, connect_data, dispatcher_url)

        return output


    @staticmethod
    @app.post("/triggers/{trigger_name}/test")
    def trigger_test(trigger_name: str, plugin_stdin: typing.Optional[models.PluginStdin] = None):
        # 外部类
        trigger_model = plugins.triggers[trigger_name]

        # 取出body
        plugin_stdin = json.dumps(plugin_stdin.dict())
        stdin_body = core.parse_stdin(plugin_stdin)

        # 获取input
        connect_data = core.extract_pointer(stdin_body, "/connection")

        output = trigger_model.test(connect_data)

        return output


    @staticmethod
    @app.post("/actions/{action_name}")
    def actions(action_name, plugin_stdin: typing.Optional[models.PluginStdin] = None):


        # 外部类
        action_model = plugins.actions[action_name]

        # 取出body
        plugin_stdin = json.dumps(plugin_stdin.dict())
        stdin_body = core.parse_stdin(plugin_stdin)

        # 获取input
        inp = core.extract_pointer(stdin_body, "/input")
        connect_data = core.extract_pointer(stdin_body, "/connection")

        # 执行　外部run 相关操作
        action_model.connection(connect_data)
        output = action_model._run(inp, connect_data)

        return output


    @staticmethod
    @app.post("/actions/{action_name}/test")
    def actions_test(action_name: str, plugin_stdin: typing.Optional[models.PluginStdin] = None):
        # 外部类
        action_model = plugins.actions[action_name]

        # 取出body
        plugin_stdin = json.dumps(plugin_stdin.dict())
        stdin_body = core.parse_stdin(plugin_stdin)

        # 获取input
        connect_data = core.extract_pointer(stdin_body, "/connection")

        output = action_model.test(connect_data)

        return output

    @staticmethod
    @app.post("/indicator_receivers/{indicator_receiver_name}")
    def indicator_receivers(indicator_receiver_name: str, plugin_stdin: typing.Optional[models.PluginStdin]):
        # 外部类
        indicator_receiver_model = plugins.indicator_receivers[indicator_receiver_name]

        # 取出body
        plugin_stdin = json.dumps(plugin_stdin.dict())
        stdin_body = core.parse_stdin(plugin_stdin)

        # 获取input
        inp = core.extract_pointer(stdin_body, "/input")
        connect_data = core.extract_pointer(stdin_body, "/connection")
        dispatcher_url = core.extract_pointer(stdin_body, "/dispatcher/url")

        # 执行　外部run 相关操作
        output = indicator_receiver_model._run(inp, connect_data, dispatcher_url)

        return output

    @staticmethod
    @app.post("/indicator_receivers/{indicator_receiver_name}/test")
    def indicator_receivers_test(indicator_receiver_name: str, plugin_stdin: typing.Optional[models.PluginStdin] = None):
        # 外部类
        indicator_receiver_model = plugins.indicator_receivers[indicator_receiver_name]

        # 取出body
        plugin_stdin = json.dumps(plugin_stdin.dict())
        stdin_body = core.parse_stdin(plugin_stdin)

        # 获取input
        connect_data = core.extract_pointer(stdin_body, "/connection")

        output = indicator_receiver_model.test(connect_data)

        return output
    
    
    @staticmethod
    @app.post("/alarm_receivers/{alarm_receiver_name}")
    def alarm_receivers(alarm_receiver_name: str, plugin_stdin: typing.Optional[models.PluginStdin]):
        # 外部类
        alarm_receiver_model = plugins.alarm_receivers[alarm_receiver_name]

        # 取出body
        plugin_stdin = json.dumps(plugin_stdin.dict())
        stdin_body = core.parse_stdin(plugin_stdin)

        # 获取input
        inp = core.extract_pointer(stdin_body, "/input")
        connect_data = core.extract_pointer(stdin_body, "/connection")
        dispatcher_url = core.extract_pointer(stdin_body, "/dispatcher/url")

        # 执行　外部run 相关操作
        output = alarm_receiver_model._run(inp, connect_data, dispatcher_url)

        return output

    @staticmethod
    @app.post("/alarm_receivers/{alarm_receiver_name}/test")
    def alarm_receivers_test(alarm_receiver_name: str, plugin_stdin: typing.Optional[models.PluginStdin] = None):
        # 外部类
        alarm_receiver_model = plugins.alarm_receivers[alarm_receiver_name]

        # 取出body
        plugin_stdin = json.dumps(plugin_stdin.dict())
        stdin_body = core.parse_stdin(plugin_stdin)

        # 获取input
        connect_data = core.extract_pointer(stdin_body, "/connection")

        output = alarm_receiver_model.test(connect_data)

        return output

    

    def runserver(self):
        uvicorn.run(app, host="0.0.0.0", port=10001)




import tarfile
import yaml
import jinja2
from typing import List, Optional, Any
import os


from .config import *


class Tools(object):

    def tarExtract(self, tar_path, target_path):
        """
        .tar.gz　提取
        """

        try:
            tar = tarfile.open(tar_path, "r:gz")
            file_names = tar.getnames()
            for file_name in file_names:
                if not os.path.exists(os.path.join(target_path, file_name)):
                    tar.extract(file_name, target_path)
                    logging.info(f"generated {file_name} ok")
            tar.close()
        except Exception as e:
            logging.error(f"SDK生成失败 - {tar_path} - {str(e)}")

    def readYaml(self, yaml_path: str) -> dict:
        # 以json格式读取yaml

        with open(yaml_path, "r", encoding="utf-8") as f:
            yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)

        return yaml_data

    def renderStrTemplate(self, data, temp: str) -> str:
        #　字符串渲染

        template = jinja2.Template(temp)
        res = template.render(data)

        return res

    def typeTrans(self, t: str, rq: str, dft: Any, enm: Optional[List]):
        # 类型转换

        strMark = lambda x: f'"{x}"' if isinstance(x, str) else x

        argType = ""

        typeMap = {
            "string": "str",
            "bytes": "str",
            "boolean": "bool",
            "float": "float",
            "date": "str",
            "object": "dict",
            "password": "str",
            "integer": "int",
            "file": "dict"
        }

        if isinstance(enm, list):
            argType = f"Literal{enm}"
        else:
            argType = typeMap.get(t)

        if not argType:

            if t.startswith("[]"):
                argType = f"Optional[List[{typeMap.get(t[2:], self.getModelName(t[2:]))}]]"
            else:
                argType = self.getModelName(t)

        if not rq and not dft:
            argType += " = None"
        else:
            if dft:
                argType += f' = {strMark(dft)}'

        return argType

    def ymlTransPy(self, data: dict) -> List:
        # yml　转 py　类型, 返回列表，以便直接渲染

        py_list = []

        if data:
            for name,v in data.items():
                # 默认　type: string, required: false
                tp = v.get("type", "string")
                rq = v.get("required", False)
                dft = v.get("default", None)
                enm = v.get("enum", None)

                assert not (isinstance(enm, list) and dft and dft not in enm), "valid fail, default value not in enum value."

                tp = self.typeTrans(tp, rq, dft, enm)

                py_list.append([name, tp])

        return py_list

    def getModelName(self, tit: str, t="") -> str:
        # 生成校验类型名字

        return (tit + t).upper()

    def writeFile(self, data: str, filepath: str):
        # 写入文件,　生成初始化文件

        with open(filepath, "w", encoding="utf-8") as f:
            f.write(data)

    def readFile(self, filepath: str):
        # 读取文件

        with open(filepath, "r", encoding="utf-8") as f:
            data = f.read()

        return data




import os
from loguru import logger
import sys
import logging

BASE_DIR = os.path.abspath(os.path.dirname(__file__))

logging.basicConfig(level=logging.DEBUG, format="%(levelname)s | %(message)s")


modelHeader = """
from pydantic import BaseModel
from typing import *

# 可自行修改增加校验精准度

"""

MODELTEMPLATE = """
class {{ className }}(BaseModel):
    {% if args %}
    {% for argName, argType in args %}
    {{ argName }}: {{ argType }}
    {% endfor %}
    {% else %}
    ...
    {% endif %}
"""

ACTIONTEMPLATE = """
from SDK.subassembly import Actions
from SDK.base import Core

from .models import {{ connModel }}, {{ inputModel }}, {{ outputModel }}


class {{ actionsName }}(Actions):

    def __init__(self):
        # 初始化
        
        self.name = "{{ name }}"
        self.inputModel = {{ inputModel }}
        self.outputModel = {{ outputModel }}
        self.connModel = {{ connModel }}


    def connection(self, data={}):
        # write your code
        ...    
    
    
    def run(self, params={}):
        # write your code
        ...

"""

TRIGGERSTEMPLATE = """
from SDK.subassembly import Triggers
from SDK.base import Core

from .models import {{ connModel }}, {{ inputModel }}, {{ outputModel }}


class {{ triggersName }}(Triggers):

    def __init__(self):
        # 初始化
        
        self.name = "{{ name }}"
        self.inputModel = {{ inputModel }}
        self.outputModel = {{ outputModel }}
        self.connModel = {{ connModel }}


    def connection(self, data={}):
        # write your code
        ...    

    def run(self, params={}):
        # write your code
        # 返回必须使用 self.send({})
        
        ...

"""

ACTIONSTESTTEMPLATE = """
{
	"version": "v1",
	"type": "action_start",
	"body": {
		"action": "{{ title }}",
		"meta": {},
		"connection": {},
		"dispatcher": null,
		"input": {}
	}
}

"""

TRIGGERSTESTTEMPLATE = """
{
	"version": "v1",
	"type": "trigger_start",
	"body": {
		"trigger": "{{ title }}",
		"meta": {},
		"connection": {},
		"dispatcher": {
			"url": "http://127.0.0.1:8001/send",
			"webhook_url": ""
		},
		"input": {}
      "enable_web": false
	}
}

"""

MAINTEMPLATE = """#!/usr/bin/env python

from SDK.plugin import Plugin
from SDK.cli import CLI

{% if actionClassees %}
import actions
{% endif %}
{% if triggerClassees %}
import triggers
{% endif %}
{% if indicatorReceiverClassees %}
import indicator_receivers
{% endif %}
{% if alarmReceiverClassees %}
import alarm_receivers
{% endif %}


# 整个程序入口


class {{ pluginName }}(Plugin):

    def __init__(self):
        super().__init__()
        
        {% for actionClass in actionClassees %}
        self.add_actions(actions.{{ actionClass }}())
        {% endfor %}

        {% for triggerClass in triggerClassees %}
        self.add_triggers(triggers.{{ triggerClass }}())
        {% endfor %}

        {% for indicatorReceiverClasse in indicatorReceiverClassees %}
        self.add_indicator_receivers(indicator_receivers.{{ indicatorReceiverClasse }}())
        {% endfor %}
        
        {% for alarmReceiverClasse in alarmReceiverClassees %}
        self.add_alarm_receivers(alarm_receivers.{{ alarmReceiverClasse }}())
        {% endfor %}


def main():

    cli = CLI({{ pluginName }}())
    cli.run()



if __name__ == '__main__':

    main()
    
"""

INITTEMPLATE = """
{% for name, className in init_list %}
from .{{ name }} import {{ className }}
{% endfor %}
"""

HELPTEMPLATE = """
# {{ name }}

## About
{{ name }}



## Connection

{% if connection %}


{% for field_name, field_data in connection.items() -%}

{% if loop.index == 1 -%}
|Name|Type|Required|Description|Default|Enum|
|----|----|--------|-----------|-------|----|
{%- endif %}
|{%- if field_data.title -%}
{{ field_data.title['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
{{ field_data.description['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|

{%- endfor %}


{% endif %}


## Actions

{% if actions %}

{% for action, actionData in actions.items() %}

### {{ action }}

---

{% for action_name,action_data in actionData.items() %}

{% if action_name == 'input' %}
#### Input

{% for field_name, field_data in action_data.items() -%}

{% if loop.index == 1 -%}
|Name|Type|Required|Description|Default|Enum|
|----|----|--------|-----------|-------|----|
{%- endif %}
|{%- if field_data.title -%}
{{ field_data.title['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
{{ field_data.description['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|


{%- endfor %}

{% endif %}


{% if action_name == 'output' %}
#### Output

{% for field_name, field_data in action_data.items() -%}

{% if loop.index == 1 -%}
|Name|Type|Required|Description|Default|Enum|
|----|----|--------|-----------|-------|----|
{%- endif %}
|{%- if field_data.title -%}
{{ field_data.title['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
{{ field_data.description['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|


{%- endfor %}


{% endif %}



{% endfor %}

{% endfor %}

{% endif %}



## Triggers

---

{% if triggers %}


{% for trigger, triggerData in triggers.items() %}

### {{ trigger }}

---

{% for trigger_name,trigger_data in triggerData.items() %}

{% if trigger_name == 'input' %}
#### Input

{% for field_name, field_data in trigger_data.items() -%}

{% if loop.index == 1 -%}
|Name|Type|Required|Description|Default|Enum|
|----|----|--------|-----------|-------|----|
{%- endif %}
|{%- if field_data.title -%}
{{ field_data.title['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
{{ field_data.description['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|


{%- endfor %}

{% endif %}


{% if action_name == 'output' %}
#### Output

{% for field_name, field_data in action_data.items() -%}

{% if loop.index == 1 -%}
|Name|Type|Required|Description|Default|Enum|
|----|----|--------|-----------|-------|----|
{%- endif %}
|{%- if field_data.title -%}
{{ field_data.title['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
{{ field_data.description['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|


{%- endfor %}


{% endif %}



{% endfor %}

{% endfor %}

{% endif %}


## Types

{% if types %}

{% for type_name, type_data in types.items() %}

### {{ type_name }}

{% for field_name, field_data in type_data.items() -%}

{% if loop.index == 1 -%}
|Name|Type|Required|Description|Default|Enum|
|----|----|--------|-----------|-------|----|
{%- endif %}
|{%- if field_data.title -%}
{{ field_data.title['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
{{ field_data.description['zh-CN'] }}
{%- else -%}
None
{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|

{%- endfor %}

{% endfor %}

{% endif %}


## 版本信息
- {{ version }}

## 参考引用
"""

INDICATORRECEIVERSTEMPLATE = """
from SDK.subassembly import IndicatorReceivers
from SDK.base import Core

from .models import {{ connModel }}, {{ inputModel }}, {{ outputModel }}


class {{ indicator_receiversName }}(IndicatorReceivers):

    def __init__(self):
        # 初始化

        self.name = "{{ name }}"
        self.inputModel = {{ inputModel }}
        self.outputModel = {{ outputModel }}
        self.connModel = {{ connModel }}


    def connection(self, data={}):
        # write your code
        ...    

    def run(self, params={}):
        # write your code
        # 返回必须使用 self.send({})

        ...

"""

INDICATORRECEIVERSTESTTEMPLATE = """
{
	"version": "v1",
	"type": "indicator_receiver_start",
	"body": {
		"receiver": "{{ title }}",
		"meta": {},
		"connection": {},
		"dispatcher": {
			"url": "http://127.0.0.1:8001/send",
			"webhook_url": ""
		},
		"input": {},
      "enable_web": false
	}
}

"""

INDICATORRECEIVERSMODELTYPES = """
class Indicator(BaseModel):
    uid: str
    type: str
    value: str
    source: str
    reputation: str
    threat_score: int
    rawed: str
    tagsed: Optional[str] = None
    status: Optional[bool] = None
    notes: Optional[str] = None
    casesed: Optional[str] = None
    created_at: str
    updated_at: str


class IndicatorDomain(BaseModel):
    uid: str
    indicator_uid: str
    primary_domain: Optional[str] = None
    admin_name: Optional[str] = None
    organization: Optional[str] = None
    admin_email: Optional[str] = None
    admin_phone: Optional[str] = None
    admin_address: Optional[str] = None
    register_at: Optional[str] = None
    renew_at: Optional[str] = None
    name_provider: Optional[str] = None
    name_servers: Optional[str] = None


class IndicatorUrl(BaseModel):
    uid: str
    indicator_uid: str
    hash: Optional[str] = None
    host: Optional[str] = None


class IndicatorIp(BaseModel):
    uid: str
    indicator_uid: str
    hostname: Optional[str] = None
    geo_country: Optional[str] = None
    geo_location: Optional[str] = None
    open_ports: Optional[str] = None


class IndicatorHash(BaseModel):
    uid: str
    indicator_uid: str
    sha256: Optional[str] = None
    sha1: Optional[str] = None
    md5: Optional[str] = None


class IndicatorEmail(BaseModel):
    uid: str
    indicator_uid: str
    primary_domain: Optional[str] = None


class IndicatorFile(BaseModel):
    uid: str
    indicator_uid: str
    filename: Optional[str] = None
    extension: Optional[str] = None
    size: Optional[str] = None
    sha256: Optional[str] = None
    sha1: Optional[str] = None
    md5: Optional[str] = None


class IndicatorHost(BaseModel):
    uid: str
    indicator_uid: str
    ip: Optional[str] = None
    mac: Optional[str] = None
    bios: Optional[str] = None
    memory: Optional[str] = None
    processors: Optional[str] = None
    os: Optional[str] = None


class IndicatorAccount(BaseModel):
    uid: str
    indicator_uid: str
    username: Optional[str] = None
    email: Optional[str] = None
    account_type: Optional[str] = None
    role: Optional[str] = None
    domain: Optional[str] = None
    organization: Optional[str] = None
"""

INDICATORRECEIVERSMODELTEMPLATE = """
class {{ className }}(BaseModel):

    indicator: Indicator
    indicator_sub: Union[IndicatorDomain, IndicatorUrl, IndicatorIp, IndicatorHash, IndicatorEmail, IndicatorFile, IndicatorHost, IndicatorAccount, None]

"""

ALARMRECEIVERSTEMPLATE = """
from SDK.subassembly import AlarmReceivers
from SDK.base import Core

from .models import {{ connModel }}, {{ inputModel }}, {{ outputModel }}


class {{ alarm_receiversName }}(AlarmReceivers):

    def __init__(self):
        # 初始化

        self.name = "{{ name }}"
        self.inputModel = {{ inputModel }}
        self.outputModel = {{ outputModel }}
        self.connModel = {{ connModel }}


    def connection(self, data={}):
        # write your code
        ...    

    def run(self, params={}):
        # write your code
        # 返回必须使用 self.send({})

        ...

"""

ALARMRECEIVERSTESTTEMPLATE = """
{
	"version": "v1",
	"type": "alarm_receiver_start",
	"body": {
		"alarm": "{{ title }}",
		"meta": {},
		"connection": {},
		"dispatcher": {
			"url": "http://127.0.0.1:8001/send",
			"webhook_url": ""
		},
		"input": {},
      "enable_web": false
	}
}

"""

ALARMRECEIVERSMODELTYPES = """
class Alarm(BaseModel):
    uid: str
    name: str
    alarm_ip: str
    sip: str
    tip: str
    source: str
    type: str
    reputation: str
    status: Optional[bool] = True
    raw: str
    created_at: str
    updated_at: str
    
"""

ALARMRECEIVERSMODELTEMPLATE = """
class {{ className }}(BaseModel):

    alarm: Alarm
    
"""










try:
    from .base import Core
    from . import chariot
except:
    from base import Core
    import chariot



"""
SDK入口
"""


class CLI(object):

    def __init__(self, plugins):
        # 所有自写脚本的类对象
        self.plugins = plugins


    def run(self):

        core = Core()

        cmd = core.get_cmd()  # command: run, http, or test

        # mains
        if cmd == 'run':
            inp = core.get_stdin()  # input
            chariot.run(inp, self.plugins)
        elif cmd == "http":
            chariot.http(self.plugins)
        elif cmd == 'test':
            inp = core.get_stdin()  # input
            chariot.test(inp, self.plugins)








"""
搜集插件各个类
"""


class Plugin(object):


    def __init__(self):

        # 插件的３个组件
        self.connection = {}
        self.actions = {}
        self.triggers = {}
        self.indicator_receivers = {}
        self.alarm_receivers = {}


    def add_connection(self, connect):
        self.connection[connect.name] = connect

    def add_actions(self, action):
        self.actions[action.name] = action

    def add_triggers(self, trigger):
        self.triggers[trigger.name] = trigger

    def add_indicator_receivers(self, indicator_receiver):
        self.indicator_receivers[indicator_receiver.name] = indicator_receiver
    
    def add_alarm_receivers(self, alarm_receiver):
        self.alarm_receivers[alarm_receiver.name] = alarm_receiver





import os
import json

from .config import *
from .tools import Tools



tools = Tools()


def generate(path: str, yml: str):
    """
    生成插件
    """

    # 验证路径，　是否是yaml文件
    yml_path = os.path.join(path, yml)
    if not any([yml.endswith(y) for y in ["yml", "yaml"]]) or not os.path.exists(yml_path):
        logging.error(f"yaml路径错误 - {yml_path}")
        return
    else:
        yaml_data = tools.readYaml(yml_path)
        logging.info(f"readed {yml}")

    plugin_name = yaml_data.get("name", "hah")
    actions_class_list = []
    triggers_class_list = []
    indicator_receivers_class_list = []
    alarm_receivers_class_list = []

    # 当前文件路劲下生成sdk
    tar_path = os.path.join(BASE_DIR, os.path.join("res", "demo.tar.gz"))
    target_path = path
    tools.tarExtract(tar_path, target_path)

    # 读取types
    types = yaml_data.get("types")
    typesTemp = ""
    if types:
        for types_name, types_data in types.items():
            typesData = {
                "className": tools.getModelName(types_name),
                "args": tools.ymlTransPy(types_data)
            }
            typesTemp += tools.renderStrTemplate(typesData, MODELTEMPLATE)


    # 读取connection
    connection = yaml_data.get("connection")
    connection = {
                "className": tools.getModelName("connection"),
                "args": tools.ymlTransPy(connection)
            }
    connTemp = tools.renderStrTemplate(connection, MODELTEMPLATE)

    # 创建tests
    tests_path = os.path.join(path, "tests")
    if not os.path.exists(tests_path):
        os.mkdir(tests_path)


    # 生成actions
    actions = yaml_data.get("actions")
    if actions:
        actions_path = os.path.join(path, "actions")
        if not os.path.exists(actions_path):
            os.mkdir(actions_path)

        actionsTemp = ""
        actionsModelTemp = ""
        actionsModelTemp += modelHeader
        actionsModelTemp += typesTemp
        actionsModelTemp += connTemp


        init_list = []

        for title, data in actions.items():

            # models
            inp = data.get("input")
            outp = data.get("output")

            actionsName = tools.getModelName(title, "Actions")
            inpClassName = tools.getModelName(title, "Input")
            outpClassName = tools.getModelName(title, "Output")

            actions_class_list.append(actionsName)
            init_list.append([
                title,
                actionsName
            ])

            inp_data = {
                "className": inpClassName,
                "args": tools.ymlTransPy(inp)
            }
            outp_data = {
                "className": outpClassName,
                "args": tools.ymlTransPy(outp)
            }

            inpTemp = tools.renderStrTemplate(inp_data, MODELTEMPLATE)
            outpTemp = tools.renderStrTemplate(outp_data, MODELTEMPLATE)

            # model主要内容
            actionsModelTemp += inpTemp
            actionsModelTemp += outpTemp

            # action
            actionData = {
                "actionsName": actionsName,
                "name": title,
                "inputModel": inpClassName,
                "outputModel": outpClassName,
                "connModel": tools.getModelName("connection"),
            }
            actionsTemp = tools.renderStrTemplate(actionData, ACTIONTEMPLATE)

            file_path = os.path.join(actions_path, f"{title}.py")
            if not os.path.exists(file_path):
                tools.writeFile(actionsTemp, file_path)
                logging.info(f"gnerated actions/{title}.py ok")

            # 生成测试文件
            file_path = os.path.join(tests_path, f"{title}.json")
            testData = tools.renderStrTemplate({"title": title}, ACTIONSTESTTEMPLATE)
            tools.writeFile(testData, file_path)
            logging.info(f"generated tests/{title}.json ok")

        # 生成__init__.py
        file_path = os.path.join(actions_path, "__init__.py")
        initData = tools.renderStrTemplate({"init_list": init_list}, INITTEMPLATE)
        tools.writeFile(initData, file_path)
        logging.info(f"generated actions/__init__.py ok")

        file_path = os.path.join(actions_path, "models.py")
        tools.writeFile(actionsModelTemp, file_path)
        logging.info(f"gnerated actions/models.py ok")

    #===

    # 生成triggers
    triggers = yaml_data.get("triggers")
    if triggers:
        triggers_path = os.path.join(path, "triggers")
        if not os.path.exists(triggers_path):
            os.mkdir(triggers_path)

        triggersTemp = ""
        triggersModelTemp = ""
        triggersModelTemp += modelHeader
        triggersModelTemp += typesTemp
        triggersModelTemp += connTemp


        init_list = []

        for title, data in triggers.items():

            # models
            inp = data.get("input")
            outp = data.get("output")

            triggersName = tools.getModelName(title, "Triggers")
            inpClassName = tools.getModelName(title, "Input")
            outpClassName = tools.getModelName(title, "Output")

            triggers_class_list.append(triggersName)
            init_list.append([
                title,
                triggersName
            ])

            inp_data = {
                "className": inpClassName,
                "args": tools.ymlTransPy(inp)
            }
            outp_data = {
                "className": outpClassName,
                "args": tools.ymlTransPy(outp)
            }

            inpTemp = tools.renderStrTemplate(inp_data, MODELTEMPLATE)
            outpTemp = tools.renderStrTemplate(outp_data, MODELTEMPLATE)

            # model主要内容
            triggersModelTemp += inpTemp
            triggersModelTemp += outpTemp

            # trigger
            triggerData = {
                "triggersName": triggersName,
                "name": title,
                "inputModel": inpClassName,
                "outputModel": outpClassName,
                "connModel": tools.getModelName("connection"),
            }
            triggersTemp = tools.renderStrTemplate(triggerData, TRIGGERSTEMPLATE)

            file_path = os.path.join(triggers_path, f"{title}.py")
            if not os.path.exists(file_path):
                tools.writeFile(triggersTemp, file_path)
                logging.info(f"gnerated triggers/{title}.py ok")

            # 生成测试文件
            file_path = os.path.join(tests_path, f"{title}.json")
            testData = tools.renderStrTemplate({"title": title}, TRIGGERSTESTTEMPLATE)
            tools.writeFile(testData, file_path)
            logging.info(f"generated tests/{title}.json ok")

        # 生成__init__.py
        file_path = os.path.join(triggers_path, "__init__.py")
        initData = tools.renderStrTemplate({"init_list": init_list}, INITTEMPLATE)
        tools.writeFile(initData, file_path)
        logging.info(f"generated triggers/__init__.py ok")


        file_path = os.path.join(triggers_path, "models.py")
        tools.writeFile(triggersModelTemp, file_path)
        logging.info(f"gnerated triggers/models.py ok")

    #===


    # 生成indicator_receivers
    indicator_receivers = yaml_data.get("indicator_receivers")
    if indicator_receivers:
        indicator_receivers_path = os.path.join(path, "indicator_receivers")
        if not os.path.exists(indicator_receivers_path):
            os.mkdir(indicator_receivers_path)

        indicator_receiversTemp = ""
        indicator_receiversModelTemp = ""
        indicator_receiversModelTemp += modelHeader
        indicator_receiversModelTemp += INDICATORRECEIVERSMODELTYPES
        indicator_receiversModelTemp += typesTemp
        indicator_receiversModelTemp += connTemp


        init_list = []

        for title, data in indicator_receivers.items():

            # models
            inp = data.get("input")

            indicator_receiversName = tools.getModelName(title, "indicator_receivers")
            inpClassName = tools.getModelName(title, "Input")
            outpClassName = tools.getModelName(title, "Output")

            indicator_receivers_class_list.append(indicator_receiversName)
            init_list.append([
                title,
                indicator_receiversName
            ])

            inp_data = {
                "className": inpClassName,
                "args": tools.ymlTransPy(inp)
            }
            outp_data = {
                "className": outpClassName,
            }

            inpTemp = tools.renderStrTemplate(inp_data, MODELTEMPLATE)
            outpTemp = tools.renderStrTemplate(outp_data, INDICATORRECEIVERSMODELTEMPLATE)

            # model主要内容
            indicator_receiversModelTemp += inpTemp
            indicator_receiversModelTemp += outpTemp

            # indicator_receiver
            indicator_receiverData = {
                "indicator_receiversName": indicator_receiversName,
                "name": title,
                "inputModel": inpClassName,
                "outputModel": outpClassName,
                "connModel": tools.getModelName("connection"),
            }
            indicator_receiversTemp = tools.renderStrTemplate(indicator_receiverData, INDICATORRECEIVERSTEMPLATE)

            file_path = os.path.join(indicator_receivers_path, f"{title}.py")
            if not os.path.exists(file_path):
                tools.writeFile(indicator_receiversTemp, file_path)
                logging.info(f"gnerated indicator_receivers/{title}.py ok")

            # 生成测试文件
            file_path = os.path.join(tests_path, f"{title}.json")
            testData = tools.renderStrTemplate({"title": title}, INDICATORRECEIVERSTESTTEMPLATE)
            tools.writeFile(testData, file_path)
            logging.info(f"generated tests/{title}.json ok")

        # 生成__init__.py
        file_path = os.path.join(indicator_receivers_path, "__init__.py")
        initData = tools.renderStrTemplate({"init_list": init_list}, INITTEMPLATE)
        tools.writeFile(initData, file_path)
        logging.info(f"generated indicator_receivers/__init__.py ok")

        file_path = os.path.join(indicator_receivers_path, "models.py")
        tools.writeFile(indicator_receiversModelTemp, file_path)
        logging.info(f"gnerated indicator_receivers/models.py ok")

    #===



    # 生成alarm_receivers
    alarm_receivers = yaml_data.get("alarm_receivers")
    if alarm_receivers:
        alarm_receivers_path = os.path.join(path, "alarm_receivers")
        if not os.path.exists(alarm_receivers_path):
            os.mkdir(alarm_receivers_path)

        alarm_receiversTemp = ""
        alarm_receiversModelTemp = ""
        alarm_receiversModelTemp += modelHeader
        alarm_receiversModelTemp += ALARMRECEIVERSMODELTYPES
        alarm_receiversModelTemp += typesTemp
        alarm_receiversModelTemp += connTemp

        init_list = []

        for title, data in alarm_receivers.items():

            # models
            inp = data.get("input")

            alarm_receiversName = tools.getModelName(title, "alarm_receivers")
            inpClassName = tools.getModelName(title, "Input")
            outpClassName = tools.getModelName(title, "Output")

            alarm_receivers_class_list.append(alarm_receiversName)
            init_list.append([
                title,
                alarm_receiversName
            ])

            inp_data = {
                "className": inpClassName,
                "args": tools.ymlTransPy(inp)
            }
            outp_data = {
                "className": outpClassName,
            }

            inpTemp = tools.renderStrTemplate(inp_data, MODELTEMPLATE)
            outpTemp = tools.renderStrTemplate(outp_data, ALARMRECEIVERSMODELTEMPLATE)

            # model主要内容
            alarm_receiversModelTemp += inpTemp
            alarm_receiversModelTemp += outpTemp

            # alarm_receiver
            alarm_receiverData = {
                "alarm_receiversName": alarm_receiversName,
                "name": title,
                "inputModel": inpClassName,
                "outputModel": outpClassName,
                "connModel": tools.getModelName("connection"),
            }
            alarm_receiversTemp = tools.renderStrTemplate(alarm_receiverData, ALARMRECEIVERSTEMPLATE)

            file_path = os.path.join(alarm_receivers_path, f"{title}.py")
            if not os.path.exists(file_path):
                tools.writeFile(alarm_receiversTemp, file_path)
                logging.info(f"gnerated alarm_receivers/{title}.py ok")

            # 生成测试文件
            file_path = os.path.join(tests_path, f"{title}.json")
            testData = tools.renderStrTemplate({"title": title}, ALARMRECEIVERSTESTTEMPLATE)
            tools.writeFile(testData, file_path)
            logging.info(f"generated tests/{title}.json ok")

        # 生成__init__.py
        file_path = os.path.join(alarm_receivers_path, "__init__.py")
        initData = tools.renderStrTemplate({"init_list": init_list}, INITTEMPLATE)
        tools.writeFile(initData, file_path)
        logging.info(f"generated alarm/__init__.py ok")

        file_path = os.path.join(alarm_receivers_path, "models.py")
        tools.writeFile(alarm_receiversModelTemp, file_path)
        logging.info(f"gnerated alarm_receivers/models.py ok")

    # ===


    # 创建入口文件　main.py
    mainData = {
        "pluginName": tools.getModelName(plugin_name, "Plugin").replace(" ", "_"),
        "actionClassees": actions_class_list,
        "triggerClassees": triggers_class_list,
        "indicatorReceiverClassees": indicator_receivers_class_list,
        "alarmReceiverClassees": alarm_receivers_class_list
    }
    file_path = os.path.join(path, "main.py")
    mainTemp = tools.renderStrTemplate(mainData, MAINTEMPLATE)
    tools.writeFile(mainTemp, file_path)
    logging.info(f"generated main.py ok")


    # 创建help.md
    helpData = yaml_data
    file_path = os.path.join(path, "help.md")
    mainTemp = tools.renderStrTemplate(helpData, HELPTEMPLATE)
    tools.writeFile(mainTemp, file_path)
    logging.info("generated help.md ok")

    # 生成util
    util_path = os.path.join(path, "util")
    if not os.path.exists(util_path):
        os.mkdir(util_path)
        logging.info("generated util ok")

    logging.info("All things done successfully ^_^")


def run(path: str, tests: str):

    main_path = os.path.join(path, "main.py")
    tests_path = os.path.join(path, tests)

    if not os.path.exists(tests_path):
        logging.error(f"请正确输入路径")


    cmd = f"python {main_path} run < {tests_path}"
    os.system(cmd)

def http(path: str):
    main_path = os.path.join(path, "main.py")


    cmd = f"python {main_path} http"
    os.system(cmd)

def test(path: str, tests: str):
    main_path = os.path.join(path, "main.py")
    tests_path = os.path.join(path, tests)

    if not os.path.exists(tests_path):
        logging.error(f"请正确输入路径")

    cmd = f"python {main_path} test < {tests_path}"
    os.system(cmd)

def tarball(path: str):

    Makefile_path = os.path.join(path, "Makefile")
    if os.path.exists(Makefile_path):
        cmd = "make tarball"
        os.system(cmd)


def mkimg(path: str):
    Makefile_path = os.path.join(path, "Makefile")
    if os.path.exists(Makefile_path):
        cmd = "make image"
        os.system(cmd)





from typing import Optional, List
from pydantic import BaseModel, ValidationError

"""
入参出参数据格式验证，除　PluginStdin外，需自行编写对应类
"""

# json测试文件
class PluginStdin(BaseModel):
    version: str
    type: str
    body: dict














import json
import traceback
from pydantic import BaseModel, ValidationError
import importlib
import sys
from jsonpointer import resolve_pointer, JsonPointerException


try:
    from . import models
except:
    import models


"""
错误处理以及一些零散的方法
"""


class CatchErr(object):

    @staticmethod
    def print_err_stack(func):
        # 打印错误堆栈

        def inner(*args, **kwargs):

            try:
                return func(*args, **kwargs)
            except:
                traceback.print_exc(file=sys.stderr)

        return inner


class Core(object):

    def get_stdin(self):

        assert not sys.stdin.isatty(), "Stdin is empty!!"

        return sys.stdin.read()


    def print_to_stdout(self, args, rt=""):

        data = json.dumps(args, ensure_ascii=False)

        sys.stdout.write(data + rt)

        sys.stdout.flush()


    def print_to_stderr(self, args):

        data = json.dumps(args, ensure_ascii=False)

        sys.stderr.write(data)

        sys.stderr.flush()


    @CatchErr.print_err_stack
    def get_cmd(self):

        cmd_list = ["run", "test", "http"]
        cmd = ''
        if len(sys.argv) >= 2:
            cmd = sys.argv[1]

        assert any([cmd == i for i in cmd_list]), f"invalid cmd: `{cmd}`"

        return cmd


    @CatchErr.print_err_stack
    def load_cls(self, module: str, model: str = None) -> object:

        mod = importlib.import_module(module)
        if model:
            cls = getattr(mod, model)
            return cls
        else:
            return mod


    def check_data(self, data: dict, model) -> BaseModel:

        try:
            cls_obj = model(**data)

            return cls_obj

        except ValidationError as e:
            raise Exception(f"{model} 验证不通过 - {e.json()}")


    @CatchErr.print_err_stack
    def complete_output(self, body, log=None, error=None) -> dict:
        """
        完整output
        """

        output_body_object = {'output': body}

        if log:
            output_body_object['log'] = log
            output_body_object['status'] = 'ok'
        if error:
            output_body_object['error'] = error
            output_body_object['status'] = 'error'

        output = {'version': 'v1', 'type': 'action_event', 'body': output_body_object}

        return output


    @CatchErr.print_err_stack
    def parse_stdin(self, stdin: str) -> dict:

        stdin_obj = json.loads(stdin)

        # 验证stdin 入参是否有误
        plugin_stdin = self.check_data(stdin_obj, models.PluginStdin)

        body_obj = plugin_stdin.body

        assert body_obj, 'Input must be has `body` attribute'

        return body_obj


    @CatchErr.print_err_stack
    def extract_pointer(self, content: dict, pointer: str = "", default=None):
        """
        dict　中提取指定数据
        无则不报错
        """

        data = None

        try:
            data = resolve_pointer(content, pointer, default)
        except JsonPointerException as e:
            self.print_to_stdout(e)

        return data


    @CatchErr.print_err_stack
    def to_model_name(self, name, put="Input"):
        """
        拼接　model　name，以用来类型验证或调用类
        model_name遵循大驼峰规范
        格式：　
            name = "action_name"
            model_name = "ActionNameInput"
        """

        model_name = "".join([i.capitalize() for i in name.split("_")]) + put

        return model_name










from setuptools import setup, find_packages
import core

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()



setup(
    # 应用名
    name = "chariot-plugin",

    # 作者
    author = "andy",

    # 作者邮箱
    author_email = "1472942893@qq.com",

    # 描述
    description = "插件生成等功能...",

    # 版本号
    version = core.VERSION,

    # 安装 当前目录下有哪些包
    packages = find_packages(),

    # 配合 MANIFEST.ni文件上传静态资源
    include_package_data = True,

    # 碳泽社区
    url = "https://www.chariots.cn/market/introduction",

    # py版本
    python_requires = '>=3',

    # 依赖
    install_requires = [
        "argparse",
        "loguru",
        "jinja2",
        "pyyaml",
        "fastapi",
        "jsonpointer",
        "uvicorn",
        "requests",
    ],

    # 入口
    entry_points={
        "console_scripts": [
            "chariot-plugin = core.main:cmdline"
        ]
    },

    # 项目的详细描述
    long_description = long_description,
    long_description_content_type = "text/markdown",

    # 许可证
    license = 'MIT',
)








try:
    from .web import Server
    from .base import Core, CatchErr
except:
    from web import Server
    from base import Core, CatchErr



"""
入口后第一步，中转各种情况
"""


core = Core()

@CatchErr.print_err_stack
def run(plugin_stdin, plugins):

    stdin_body = core.parse_stdin(plugin_stdin)


    # 判断是 actions 还是 triggers
    action_name = core.extract_pointer(stdin_body, "/action")
    trigger_name = core.extract_pointer(stdin_body, "/trigger")
    indicator_receiver_name = core.extract_pointer(stdin_body, "/receiver")
    alarm_receiver_name = core.extract_pointer(stdin_body, "/alarm")

    connect_data = core.extract_pointer(stdin_body, "/connection")

    if action_name:
        """
        actions
        plugin_spec_version: v1
        走脚本
        """

        # 外部类
        action_model = plugins.actions[action_name]

        # 获取input
        inp = core.extract_pointer(stdin_body, "/input")

        # 执行　外部run 相关操作
        action_model._run(inp, connect_data)

    elif trigger_name:

        enable_web = core.extract_pointer(stdin_body, "/enable_web")

        if enable_web:
            """
            triggers
            body.enable_web: True
            跑web服务
            """
            server = Server(plugins)
            server.runserver()

        else:
            """
            triggers
            body.enable_web: False
            走脚本
            """

            # 外部类
            trigger_model = plugins.triggers[trigger_name]

            # 获取input
            inp = core.extract_pointer(stdin_body, "/input")
            dispatcher_url = core.extract_pointer(stdin_body, "/dispatcher/url")

            # 执行　外部run 相关操作
            trigger_model._run(inp, connect_data, dispatcher_url)

    elif indicator_receiver_name:

        enable_web = core.extract_pointer(stdin_body, "/enable_web")

        if enable_web:
            """
            indicator_receiver
            body.enable_web: True
            跑web服务
            """
            server = Server(plugins)
            server.runserver()

        else:
            """
            indicator_receivers
            body.enable_web: False
            走脚本
            """

            # 外部类
            indicator_receiver_model = plugins.indicator_receivers[indicator_receiver_name]

            # 获取input
            inp = core.extract_pointer(stdin_body, "/input")
            dispatcher_url = core.extract_pointer(stdin_body, "/dispatcher/url")

            # 执行　外部run 相关操作
            indicator_receiver_model._run(inp, connect_data, dispatcher_url)

    elif alarm_receiver_name:

        enable_web = core.extract_pointer(stdin_body, "/enable_web")

        if enable_web:
            """
            alarm_receiver
            body.enable_web: True
            跑web服务
            """
            server = Server(plugins)
            server.runserver()

        else:
            """
            alarm_receivers
            body.enable_web: False
            走脚本
            """

            # 外部类
            alarm_receiver_model = plugins.alarm_receivers[alarm_receiver_name]

            # 获取input
            inp = core.extract_pointer(stdin_body, "/input")
            dispatcher_url = core.extract_pointer(stdin_body, "/dispatcher/url")

            # 执行　外部run 相关操作
            alarm_receiver_model._run(inp, connect_data, dispatcher_url)


@CatchErr.print_err_stack
def test(plugin_stdin, plugins):

    stdin_body = core.parse_stdin(plugin_stdin)

    connect_data = core.extract_pointer(stdin_body, "/connection")

    name = ""
    action_name = core.extract_pointer(stdin_body, "/action")
    trigger_name = core.extract_pointer(stdin_body, "/trigger")
    indicator_receiver_name = core.extract_pointer(stdin_body, "/indicator_receiver")
    alarm_receiver_name = core.extract_pointer(stdin_body, "/alarm_receiver")

    if action_name:
        name = action_name
        model = plugins.actions[name]
    elif trigger_name:
        name = trigger_name
        model = plugins.triggers[name]
    elif indicator_receiver_name:
        name = indicator_receiver_name
        model = plugins.indicator_receivers[name]
    elif alarm_receiver_name:
        name = alarm_receiver_name
        model = plugins.alarm_receivers[name]

    if name:
        output = model.test(connect_data)
        return output



@CatchErr.print_err_stack
def http(plugins):
    server = Server(plugins)
    server.runserver()


